require "date"

# An Earth Decimal Time class
# Is a Comparable and Rangeable (ranges are in whole ticks)
class EDTDate
	include Comparable
	attr_accessor :year, :day, :ticks

	# Creates a new EDTDate.  By default, the date is NOW.
	# Argument may be a Time, a String, a Number or nil.  If String,
	# it is a properly formatted EDT date.  If Number, is expected
	# to be Ticks since the epoch
	def initialize( time=Time.new )
		if time.is_a? String
			parse( time )
		elsif time.is_a? Time
			from_time( time )
		elsif time.is_a? Numeric
			self.tepoch = time
		end
	end

	def <=>( other )
		if other.is_a? EDTDate
			self.tepoch <=> other.tepoch
		elsif other.is_a? Time
			self.tepoch <=> epoch_to_tepoch(other.to_f)
		end
	end

	def succ
		new_date = EDTDate.new nil
		new_date.year = self.year
		new_date.day = self.day
		new_date.ticks = self.ticks+1
		new_date
	end

	# Other is the number of ticks to add to this date.
	def +(other)
		if other.is_a? Numeric
			EDTDate.new(self.tepoch + other)
		end
	end

	# If other is an EDTDate, returns the number of ticks difference.
	# If other is a number, subtracts other ticks from current date
	def -(other)
		if other.is_a? EDTDate
			self.tepoch - other.tepoch
		elsif other.is_a? Numeric
			EDTDate.new(self.tepoch - other)
		end
	end

	def to_time
		Time.at(self.tepoch * 86.4)
	end

	def to_s
		"*#{year}-"+("%03d"%day)+"+"+("%3.3f"%ticks)
	end

	def date
		[self.year, self.day, self.ticks]
	end

	# The time in ticks since the epoch
	def tepoch
		ttl_ticks = (365*(self.year-1970))+((self.year-1969)/4)
		ttl_ticks += self.day-1
		ttl_ticks *= 1000
		ttl_ticks += self.ticks
	end

	# Value is ticks since the epoch
	def tepoch=( value )
		time = Time.at value * 86.4
		time.utc
		date = Date.new time.year, time.month, time.mday
		self.year = date.year
		self.day = date.yday
		self.ticks = ticks_of_tepoch( value )
	end

	private

	def epoch_to_tepoch( seconds )
		seconds / 86.4
	end
	
	def ticks_of_tepoch( tepoch ) 
		((tepoch * 1000) % 1000000)/1000.0
	end
	
	def parse( string )
		time = Time.new
		date = Date.new time.year, time.month, time.mday
		year, day, ticks, mt = string.scan(/(\*[0-9]+)?(\-[0-9]+)?(\+[0-9]+)?(\.[0-9]+)?/)[0]
		self.year = year.nil? ? date.year : year[1..-1].to_i
		self.day = day.nil? ? date.yday : day[1..-1].to_i
		mt = mt.nil? ? 0.0 : mt.to_f
		if not ticks.nil?
			self.ticks = ticks[1..-1].to_f + mt
		else 
			self.ticks = ticks_of_tepoch(epoch_to_tepoch( time.to_f ))
		end
	end

	def from_time( time )
		self.tepoch = time.to_f / 86.4
	end
end
